/*
 * libbsr - allocation services for the Barrier Synchronization Register
 *          on IBM POWER systems
 *
 * Copyright (C) 2008 Sonny Rao, IBM Corporation.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
#ifndef _BSR_H_
#define _BSR_H_

#include <stdint.h>

/* total_bytes -- 
      Total bytes of the BSR facility currently present within the system 
      or logical partition
   supported_window_mask -- 
      A Bit mask representing supported power-of-2-sized 
      windows that can be allocated
   free_bytes -- 
      Number of BSR bytes currently available (not allocated)
   free_window_-- 
       A Bit mask representing available (not allocated) 
       power-of-2-sized windows*/
/* returns 0 on success, ENODEV if no BSR is present */

int bsr_query (unsigned *total_bytes, uint32_t *supp_window_mask, 
	       unsigned *free_bytes, uint32_t *free_window_mask);

/* try to allocate a bsr of the specified size bsr_bytes */
/* bsr_stride is the interval at which the bsr repeats in the mapped region*/
/* bsr_id is a token to be passed to bsr_map and bsr_free */
/* returns 0 on success, EBUSY if there are no bsr_bytes size available */
int bsr_alloc (unsigned bsr_bytes, unsigned *bsr_stride, unsigned *bsr_id);

/* free a bsr device */
int bsr_free(unsigned bsr_id);

/* bsr_map, creates a memory map to the bsr device specified using mmap(2) */
/* prot and flags are passed to mmap, if 0 then sane defaults are used */
/* bsr_length is the size of the mmaped region */
void *bsr_map(void *addr, unsigned bsr_id, int prot, int flags, 
	      unsigned *bsr_length);

#endif /* _BSR_H_ */
