/*
 * libbsr - allocation services for the Barrier Synchronization Register
 *          on IBM POWER systems
 *
 * Copyright (C) 2008 Sonny Rao, IBM Corporation.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "bsr.h"


int main(void)
{
	unsigned i, total, smask, free,fmask, bsr_stride = 0;
	unsigned id, bsr_length, alloc_size = 0;
	void * map;	
	int ret = bsr_query(&total, &smask, &free , &fmask );
	printf ("total = %d\n"
		"smask = 0x%x\n"
		"free  = %d\n"
		"fmask = 0x%x\n",
		total, smask, free, fmask);
	if (!fmask) {
		printf("no free BSRs\n");
		return 0;
	}

	/* find the smallest BSR and allocate it */
	for (i = 0 ; i < (sizeof(fmask) * CHAR_BIT) ;i++) {
		unsigned cur_size = 1 << i;
		if (cur_size & fmask) {
			alloc_size = cur_size;
			break;
		}		
	}
	printf("trying to alloc a %d  byte BSR\n", alloc_size);
	ret = bsr_alloc(alloc_size, &bsr_stride, &id);
	printf("error = %s\n", strerror(ret));
	printf("ret = %d\n", ret);
	printf("stride = %d id = 0x%x\n", bsr_stride, id);
	if (!ret) {
		map = bsr_map(NULL, id, 0,0, &bsr_length);
		bsr_free(id);
	}
	return 0;
}
